/***************************************************************************
                          ProgressDialog.cpp  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// QT Includes
#include <qfile.h>
#include <qfileinfo.h>
#include <qlayout.h>
#include <qmultilineedit.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qtooltip.h>

// KDE includes
#include <kapplication.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <krun.h>

// Own includes
#include "ProgressDialog.h"
#include "wizard.h"

ProgressDialog::ProgressDialog( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    resize( 400, 380 );
    setCaption( i18n("Progress") );
    setIcon( BarIcon( "krename" ) );

    list = new QStrList();

    ProgressDialogLayout = new QHBoxLayout( this, 11, 6 );
    Layout1 = new QVBoxLayout( 0, 0, 6 );
    Layout3 = new QHBoxLayout( 0, 0, 6 );
    Layout2 = new QVBoxLayout( 0, 0, 6 );

    lineEdit = new QMultiLineEdit( this );
    lineEdit->setAutoUpdate( false );
    lineEdit->setReadOnly( true );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );

    buttonSave = new KPushButton( this );
    buttonSave->setText( i18n( "&Save As..." ) );
    QToolTip::add( buttonSave, i18n("Save output in a text file.") );

    buttonClose = new KPushButton( this );
    buttonClose->setText( i18n( "&Close" ) );

    buttonRestart = new KPushButton( this );
    buttonRestart->setText( i18n( "&Rename More" ) );
    QToolTip::add( buttonRestart, i18n("Restarts krename") );

    buttonAgain = new KPushButton( this );
    buttonAgain->setText( i18n( "Rename Files &Again") );
    QToolTip::add( buttonAgain, i18n( "Rename output files again" ) );

    buttonOpenDest = new KPushButton( this );
    buttonOpenDest->setText( i18n("&Open Destination") );

    Layout1->addItem( spacer );
    Layout1->addWidget( lineEdit );
    Layout3->addItem( spacer_2 );
    Layout3->addWidget( buttonSave );
    Layout3->addWidget( buttonClose );
    Layout3->addWidget( buttonRestart );
    Layout3->addWidget( buttonAgain );
    Layout3->addWidget( buttonOpenDest );
    Layout3->addItem( spacer_3 );


    Layout2->addLayout( Layout1 );
    Layout2->addLayout( Layout3 );
    ProgressDialogLayout->addLayout( Layout2 );

    connect( buttonSave, SIGNAL(clicked()), this, SLOT(save()));
    connect( buttonRestart, SIGNAL(clicked()), this, SLOT(restart() ));
    connect( buttonClose, SIGNAL(clicked()), this, SLOT(quitAll()));
    connect( buttonAgain, SIGNAL(clicked()), this ,SLOT(again()));
    connect( buttonOpenDest, SIGNAL(clicked()), this, SLOT(openDest()));

    hide();
}

ProgressDialog::~ProgressDialog()
{
    delete renamedFiles;
}

void ProgressDialog::print( QString text )
{
    list->append( text );
    kapp->processEvents( 0 );
    return;
}

void ProgressDialog::error( QString text )
{
    print( QString( i18n("ERROR: %1") ).arg(text) );
}

void ProgressDialog::warning( QString text )
{
    print( QString( i18n("WARNING: %1") ).arg(text) );
}

void ProgressDialog::quitAll()
{
    KApplication::exit(0);
}

void ProgressDialog::save()
{
    QString f = KFileDialog::getSaveFileName( QString::null, "*", this );
    if ( f.isEmpty() )
        return;

    QFile* file = new QFile( f );
    if( !file->open( IO_WriteOnly ) )
        return;

    QTextStream t( file );
    t << lineEdit->text();

    file->close();

    print("");
    print( QString( i18n("Saved output to file %1.") ).arg(f) );
    print("");
}

void ProgressDialog::done()
{
    QString tmp, text;
    for ( tmp=list->first(); tmp != 0; tmp=list->next())
        text.append( "\n" ).append( tmp );

    list->clear();
    lineEdit->append( text );
    lineEdit->setAutoUpdate( true );
    lineEdit->update();
    show();
}

void ProgressDialog::show()
{
    QWidget::show();
    
    int w = kapp->desktop()->width();
    int h = kapp->desktop()->height();
    move( (w-width())/2,(h-height())/2 );

}

void ProgressDialog::reject()
{
    // Do nothing
}

void ProgressDialog::restart()
{
    wizard* w = new wizard();
    qApp->setMainWidget( w );
    w->show();
    delete this;
}

void ProgressDialog::again()
{
    wizard* w = new wizard();
    qApp->setMainWidget( w );

    for( unsigned int i = 0; i < renamedFiles->count(); i++ ) {
        QFileInfo fi( (*renamedFiles)[i] );
        if( !fi.isDir() )
            w->addFile( (*renamedFiles)[i] );
        else
            w->addDirName( (*renamedFiles)[i] );
    }
    w->show();
    w->updatePre();
    delete this;
}

void ProgressDialog::setRenamedFiles( QStringList* list )
{
    renamedFiles = list;
}

void ProgressDialog::openDest()
{
    if(!dest.isEmpty() )
        new KRun( dest );
}
