/***************************************************************************
                          confdialog.cpp  -  description
                             -------------------
    begin                : Sun Jan 27 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Own includes
#include "confdialog.h"
#include "pluginloader.h"
#include "fileoperation.h"

// QT includes
#include <qbuttongroup.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qtooltip.h>

// KDE includes
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>
#include <kprocess.h>
#include <kstddirs.h>

ConfDialog::ConfDialog( PluginLoader* pa, QWidget* parent, const char* name, bool modal, WFlags fl )
    :  KDialogBase( KDialogBase::IconList, "KRename",
       KDialogBase::Ok | KDialogBase::Cancel | KDialogBase::Default, KDialogBase::Ok, parent, name )
{
    p = pa;

    setupTab1();
    setupTab2();
    setupTab3();

    QDictIterator<PluginLoader::PluginLibrary> it( p->libs ); // iterator for dict
    for( ; it.current(); ++it ) {
        QString i = it.currentKey();
        if( p->libs[i]->isActive )
            visiblePlugins->insertItem( p->libs[i]->plugin->getName(), -1 );
        else
            invisiblePlugins->insertItem( p->libs[i]->plugin->getName(), -1 );
    }

    connect( this, SIGNAL( defaultClicked() ), this, SLOT( defaults() ) );
}

ConfDialog::~ConfDialog()
{
}

void ConfDialog::setupTab1()
{
    QLabel* la;
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    QFrame* tab1 = addPage( i18n("Plugins"), "", BarIcon("konqueror") );
    QHBoxLayout* tabLayout = new QHBoxLayout( tab1, 11, 6 );

    QVBoxLayout* Layout1 = new QVBoxLayout( 0, 6, 6 );
    QVBoxLayout* Layout2 = new QVBoxLayout( 0, 6, 6 );
    QVBoxLayout* Layout3 = new QVBoxLayout( 0, 6, 6 );

    la = new QLabel( i18n( "Visible Plugins" ), tab1 );
    Layout1->addWidget( la );
    visiblePlugins = new QListBox( tab1 );
    Layout1->addWidget( visiblePlugins );

    bAdd = new KPushButton( tab1 );
    bAdd->setPixmap( BarIcon("1leftarrow") );
    bDel = new KPushButton( tab1 );
    bDel->setPixmap( BarIcon("1rightarrow") );
    
    Layout2->addItem( spacer );
    Layout2->addWidget( bAdd );
    Layout2->addWidget( bDel );
    Layout2->addItem( spacer );

    la = new QLabel( i18n( "Invisible Plugins" ), tab1 );
    Layout3->addWidget( la );
    invisiblePlugins = new QListBox( tab1 );

    Layout3->addWidget( invisiblePlugins );

    tabLayout->addLayout( Layout1 );
    tabLayout->addLayout( Layout2 );
    tabLayout->addLayout( Layout3 );

    connect( bAdd, SIGNAL(clicked()), this, SLOT(add()));
    connect( bDel, SIGNAL(clicked()), this, SLOT(del()));
}

void ConfDialog::setupTab2()
{
    QFrame* box = addPage( i18n("KRename"), "", BarIcon("krename") );
    QVBoxLayout* layout = new QVBoxLayout( box );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    checkContext = new QCheckBox( i18n("&Add KRename to Konquerors context menu"), box );     
    checkTitle = new QCheckBox( i18n("&Enable Advanced Titlebar"), box );
    checkAsk = new QCheckBox( i18n("Ask for &Confirmation on Exit"), box );
    checkDir = new QCheckBox( i18n("Add &Directory Names to the List with its Filenames"), box );
    checkHiddenDir = new QCheckBox( i18n("Add &Hidden Directories"), box );
    checkPlugins = new QCheckBox( i18n("&Load File Plugins on Start"), box );

    spinSize = new KIntNumInput( box );
    spinSize->setRange( 20, 500, 1, false );
    spinSize->setLabel( i18n( "Thumbnail Size:" ), AlignLeft | AlignVCenter );
    
    layout->addWidget( checkContext );
    layout->addWidget( checkTitle );
    layout->addWidget( checkAsk );
    layout->addWidget( checkDir );
    layout->addWidget( checkHiddenDir );
    layout->addWidget( checkPlugins );
    layout->addItem( spacer );
    layout->addWidget( spinSize );
    layout->addItem( spacer );

    checkContext->setChecked( serviceEntryInstalled() );
    
    connect( checkContext, SIGNAL( toggled(bool) ), this, SLOT( changeService(bool) ));

    QToolTip::add( checkHiddenDir, i18n("If not checked, KRename will ignore directories starting with a dot during recursive adding.") );
    QToolTip::add( checkPlugins, i18n("Disabling this option increases KRename's startup time, because no KFilePlugins are loaded.") );
}

void ConfDialog::setupTab3()
{
    QFrame* box = addPage( i18n("File Extension"), "", BarIcon("unknown") );

    QVBoxLayout* layout = new QVBoxLayout( box );
    QButtonGroup* group = new QButtonGroup( box );
    group->setColumnLayout(0, Qt::Vertical );
    QGridLayout* lgroup = new QGridLayout( group->layout() );
    
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    
    optionFirstDot = new QRadioButton( group );
    optionFirstDot->setText( i18n( "&first dot" ) );

    optionLastDot = new QRadioButton( group );
    optionLastDot->setText( i18n( "&last dot" ) );

    optionCustomDot = new QRadioButton( group );
    optionCustomDot->setText( i18n( "&custom dot" ) );

    spinDot = new KIntNumInput( group );
    spinDot->setRange( 1, 100, 1, false );
    spinDot->setEnabled( optionCustomDot->isChecked() );

    lgroup->addWidget( new QLabel( i18n( "File extension starts at..." ), group ), 0, 0 );
    lgroup->addWidget( optionFirstDot, 1, 0 );
    lgroup->addWidget( optionLastDot, 2, 0 );
    lgroup->addWidget( optionCustomDot, 3, 0 );
    lgroup->addWidget( spinDot, 3, 1 );
    
    layout->addWidget( group );
    layout->addItem( spacer );

    QToolTip::add( optionFirstDot, i18n("If the filename is file.tar.bz2, .tar.bz2 is used as file extension." ));
    QToolTip::add( optionLastDot, i18n("If the filename is file.tar.bz2, .bz2 is used as file extension." ));
    QToolTip::add( optionCustomDot, i18n("The file extension starts at the given dot." ));
    QToolTip::add( spinDot, i18n("The file extension starts at the given dot." ));

    connect( optionFirstDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionLastDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionCustomDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
}

void ConfDialog::accept()
{
    QDictIterator<PluginLoader::PluginLibrary> it( p->libs ); // iterator for dict
    for( ; it.current(); ++it ) {
        QString i = it.currentKey();
        p->libs[i]->isActive = false;

        QString name = p->libs[i]->plugin->getName();
        for( unsigned int z = 0; z < visiblePlugins->count(); z++ )
            if( name == visiblePlugins->text(z) )
                p->libs[i]->isActive = true;
    }

    KDialogBase::accept();
}

void ConfDialog::add()
{
    unsigned int z;
    for( z = 0; z < invisiblePlugins->count(); z++ )
        if( invisiblePlugins->isSelected( z ) ) {
            visiblePlugins->insertItem( invisiblePlugins->text( z ) );
            invisiblePlugins->removeItem( z );
        }
    invisiblePlugins->setSelected( invisiblePlugins->currentItem(), true );
}

void ConfDialog::del()
{
    unsigned int z;
    for( z = 0; z < visiblePlugins->count(); z++ )
        if( visiblePlugins->isSelected( z ) ) {
            invisiblePlugins->insertItem( visiblePlugins->text( z ) );
            visiblePlugins->removeItem( z );
        }
    visiblePlugins->setSelected( visiblePlugins->currentItem(), true );
}


void ConfDialog::installServiceEntry()
{
    QString src = locate( "data", "krename/krenameservicemenu.desktop" );
    QString dest = locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );

    FileOperation f;
    if( f.fcopy((const char*) src, (const char*) dest ) )
        KMessageBox::information( this, i18n("Added KRename to your service menu.") );
    else
        KMessageBox::sorry( this, i18n("Can't add KRename to your service menu.") );
}

void ConfDialog::removeServiceEntry()
{
    // Remove local version
    QString dest = locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );
    if( KStandardDirs::exists( dest )  )
        if( !QFile::remove( dest ) ) {
            KMessageBox::sorry( this, i18n("Can't remove service menu entry!") );
        } else
            KMessageBox::information( this, i18n("KRename was succesfully removed from your service menu!") );

    // Check for a non local version
    dest = locate( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );
    if( !dest.isEmpty() ) {
        KMessageBox::information( this, i18n(
            "<qt>KRename found another service menu entry in :<br><br>") + dest +
            i18n("<br><br>KRename requires your super user password to remove it.</qt>") );


        KProcess* p = new KProcess();
        *p << "kdesu" << "rm" << dest;
        p->start();
    }
}

bool ConfDialog::serviceEntryInstalled()
{
    if( !KStandardDirs::exists(
        locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" ) )  &&
        locate( "data", "konqueror/servicemenus/krenameservicemenu.desktop" ).isEmpty() )
        return false;

    return true;

}

void ConfDialog::changeService(bool b)
{
    if( b )
        installServiceEntry();
    else
        removeServiceEntry();

    checkContext->setChecked( serviceEntryInstalled() );
}

void ConfDialog::enableControls()
{
    spinDot->setEnabled( optionCustomDot->isChecked() );
}

void ConfDialog::defaults()
{
    optionFirstDot->setChecked( true );
    spinDot->setValue( 1 );

    checkTitle->setChecked( true );
    checkAsk->setChecked( true );
    checkDir->setChecked( false );
    checkHiddenDir->setChecked( false );
    checkPlugins->setChecked( true );

    spinSize->setValue( 80 );

    enableControls();
}

