/***************************************************************************
                          fileplugin.cpp  -  description
                             -------------------
    begin                : Mon Jul 1 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fileplugin.h"

// Qt includes
#include <qcheckbox.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qscrollview.h>
#include <qstringlist.h>
#include <qwidget.h>
#include <qvbox.h>

// KDE includes
#include <kapplication.h>
#include <kfilemetainfo.h>
#include <klineedit.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kservice.h>
#include <kmimetype.h>

FilePlugin::FilePlugin( KService* service )
{
    mip = KFileMetaInfoProvider::self();
    name = service->name();
    comment = service->comment();
    QStringList options = service->serviceTypes();
    for( unsigned int i = 0; i < options.count(); i++ ) {
        if( options[i] != "KFilePlugin" ) {
            mimetype = options[i];
            keys = mip->preferredKeys( mimetype );
            groups = mip->preferredGroups( mimetype );
            fileplugin = mip->plugin( mimetype );

            KMimeType::Ptr mime = KMimeType::mimeType( mimetype );
            icon = mime->icon( QString::null, true );  // arguments are unused
        }
    }
}

FilePlugin::~FilePlugin()
{ }

QString FilePlugin::getName()
{
    return name;
}

QString FilePlugin::getAccelName()
{
    return "&" + getName();
}

int FilePlugin::type()
{
    return TYPE_ALL;
}

bool FilePlugin::checkError()
{
    return true;
}

void FilePlugin::drawInterface( QWidget* w, QVBoxLayout* l )
{
    QSpacerItem* spacer = new QSpacerItem( 200, 20, QSizePolicy::Expanding, QSizePolicy::Expanding );

    l->addWidget( new QLabel( i18n("<qt><small>This is a KFilePlugin running in Krename's KFilePlugin Wrapper.</small></qt>"), w ) );

    QGridLayout* grid = new QGridLayout();
    QHBoxLayout* hbox = new QHBoxLayout( 0, 6, 6 );
    
    QLabel* pix = new QLabel( w );
    pix->setPixmap( kapp->iconLoader()->loadIcon( icon, KIcon::Desktop ) );

    buttonWrite = new KPushButton( i18n("&Write Meta-Info"), w );

    hbox->addWidget( pix );
    hbox->addWidget( new QLabel( "<qt><b>"+getName()+"</b></qt>", w ) );
    hbox->addItem( spacer );

    l->addLayout( hbox );    
    l->addWidget( new QLabel( comment, w ) );
    QString text = i18n("Supported tokens:<br>");
    l->addWidget( new QLabel( text, w ) );

    int x = 0;
    int y = 0;
    for( unsigned int i = 0; i < keys.count(); i++ ) {
        grid->addWidget( new QLabel( "<b>[" + keys[i] + "]</b>", w ), x, y );
        if( y < 4 )
            y++;
        else {
            x++;
            y = 0;
        }
    }

    QHBoxLayout* hb2 = new QHBoxLayout( 0, 6, 6 );
    hb2->addWidget( buttonWrite );
    hb2->addItem( spacer );

    l->addLayout( grid );
    l->addLayout( hb2 );
    l->addItem( spacer );
    l->setStretchFactor( grid, 2 );

    connect( buttonWrite, SIGNAL( clicked() ), this, SLOT( changeMetaInfo() ) );
}

void FilePlugin::fillStructure()
{
}

QString FilePlugin::processFile( values* val, int i, QString token, int mode )
{
    QString filename = val->files[i].source_dir + val->files[i].source_name + val->files[i].extension;

    if( mode == TYPE_BRACKET ) {
        for( unsigned int i = 0; i < keys.count(); i++ ) {
            if( token.lower() == keys[i].lower() ) {
                KFileMetaInfo meta( filename );
                if( meta.isValid() )
                    return meta.item( keys[i] ).string( true );
            }
        }
    } else if( mode == TYPE_FINAL_FILE ) {
        for( unsigned int i = 0; i < keys.count(); i++ ) {
            if( map.contains( keys[i] ) ) {
                KFileMetaInfo meta( token );
                if( !meta.isValid() )
                    continue;

                if(!meta.item( keys[i] ).setValue( map[keys[i]] ))
                    qDebug("Could not write meta info to: " + filename );

                if(!meta.applyChanges())
                    qDebug("Aplying failed");
            }
        }

        return "";
    } else if( mode == TYPE_TOKEN ) {
        // even if you do not look for a token
        // return the argument!
        return token;
    }

    return "";
}

void FilePlugin::finished()
{
}

bool FilePlugin::defaultVisible()
{
    return false;
}

void FilePlugin::addHelp( HelpDialog* combo )
{
    combo->addHeadline( getName() );
    for( unsigned int i = 0; i < keys.count(); i++ )
        combo->add( "[" + keys[i] + "]", keys[i] );
}

void FilePlugin::removeHelp( HelpDialog* combo )
{
    combo->removeHeadline( getName() );
    for( unsigned int i = 0; i < keys.count(); i++ )
        combo->remove( "[" + keys[i] + "]" );
}

void FilePlugin::changeMetaInfo()
{
    MetaInfoDialog* mid = new MetaInfoDialog( 0 );
    mid->setKeys( &keys, fileplugin, mimetype );
    for( unsigned int i = 0; i < keys.count(); i++ )
        if( map.contains( keys[i] ) )
            for( unsigned int z = 0; z < mid->value.size(); z++ )
                if( mid->label[z]->text() == keys[i] ) {
                    mid->check[z]->setChecked( true );
                    mid->value[z]->setText( map[keys[i]] );
                }

    mid->enableLineEdits();

    if( mid->exec() == QDialog::Accepted ) {
        map.clear();
        for( unsigned int i = 0; i < mid->value.size(); i++ ) {
            if( mid->value[i]->isEnabled() )
                map.insert( mid->label[i]->text(), mid->value[i]->text() );
        }

    }
}

MetaInfoDialog::MetaInfoDialog( QWidget* parent )
    : KDialogBase( KDialogBase::Plain, "KRename",
      KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok, parent )
{
    QVBoxLayout* layout = new QVBoxLayout( plainPage(), 6, 6 );

    view = new QScrollView( plainPage() );

    layout->addWidget( view );

    resize( 400, 400 );
}

MetaInfoDialog::~MetaInfoDialog()
{

}

void MetaInfoDialog::setKeys( QStringList* k, KFilePlugin* p, QString mimetype )
{
    QWidget* big_box = new QWidget(view->viewport());
    view->addChild(big_box);

    check.resize( k->count() );
    value.resize( k->count() );
    label.resize( k->count() );

    QVBoxLayout* v = new QVBoxLayout( big_box );
    for( unsigned int i = 0; i < k->count(); i++ ) {
        QHBoxLayout* l = new QHBoxLayout( 0, 6, 6 );

        check[i] = new QCheckBox( i18n("Enable "), view->viewport() );
        label[i] = new QLabel( (*k)[i], view->viewport() );
        value[i] = new KLineEdit( view->viewport() );

        QValidator* val = p->createValidator( mimetype, QString::null, (*k)[i], 0, (*k)[i]);

        value[i]->setEnabled( false );
        value[i]->setValidator( val );

        connect( check[i], SIGNAL( clicked() ), this, SLOT( enableLineEdits() ) );

        l->addWidget( check[i] );
        l->addWidget( label[i] );
        l->addWidget( value[i] );

        v->addLayout( l );
    }
}

void MetaInfoDialog::enableLineEdits()
{
    for( unsigned int i = 0; i < check.size(); i++ )
        value[i]->setEnabled( check[i]->isChecked() );
}
