/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your2192 option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kapplication.h>
#include <kaboutapplication.h>
#include <kcmdlineargs.h>
#include <kimageio.h>
#include <klocale.h>
#include <kmessagebox.h>

// OS includes
#include <unistd.h>
#include <sys/types.h>

// Own includes
#include "krename.h"
#include "wizard.h"
#include <config.h>

KAboutData aboutData( "krename", I18N_NOOP("Krename"), VERSION, I18N_NOOP(
    "Krename is a batch file renamer which can rename a\n"
    "list of files based on a set of expressions.\n\n"
    "If you like krename you may want to support it.\n"
    "Testing, bug fixes and feature request are as welcome\n"
    "as financial support (everybody needs money ;)\nSee help files for details.\n" ),
    KAboutData::License_GPL, "Krename Build:" __DATE__ " " __TIME__
    , "(c) 2001-2003, Dominik Seichter\n",
    "http://www.krename.net", "domseichter@web.de" );


static KCmdLineOptions options[] =
{
    { "+[file]", I18N_NOOP("file will be added to the list of files for renaming"), 0},
    { "r +[dir]", I18N_NOOP("add directory recursively"), 0 },
    { "nopreview",  I18N_NOOP("create no realtime preview"), 0 }
};


int main(int argc, char *argv[])
{
    int i;
    unsigned int uid;
    wizard* krename;

    aboutData.addAuthor("Dominik Seichter", 0, "domseichter@web.de",
        "http://www.krename.net" );
    aboutData.addAuthor("Stefan \"Stonki\" Onken",
        I18N_NOOP("Website, testing, very good ideas and keeping me coding!"),
        "stonki@gmx.de", "http://www.stonki.de" );

    aboutData.addCredit("Trevor Semeniuk",  I18N_NOOP("Thanks to him for creating RedHat 7.x packages and some other help."),
                        "semeniuk@ee.ualberta.ca", "http://www.semeniuk.net" );
    aboutData.addCredit("Groult Richard",   I18N_NOOP("Fixed a bug with startIndex and added the BatchRenamer class\n"
                        "to his excellent imager viewer showimg."),
                        "rgroult@jalix.org", "http://ric.jalix.org/" );
    aboutData.addCredit("Michael Elvers",  I18N_NOOP("Fixed a bug that caused krename not closing open files."),
                        "m_elvers@yahoo.com", "http://come.to/melvers" );
    aboutData.addCredit("Andreas Pour",  I18N_NOOP("Thanks for his great job at apps.kde.com and help with contributing krename to apps.kde.com."),
                        "pour@mieterra.com", "http://apps.kde.com" );
    aboutData.addCredit("Charles Samuels", I18N_NOOP("Thanks for noatun and the ID3/Ogg Tag code is based on his noatun modules."),
                        "charles@kde.org", "http://noatun.kde.org/" );
    aboutData.addCredit("Franz Schmid", I18N_NOOP("Gave me a good start into writing plugins with his application scribus."),
                        "Franz.Schmid@altmuehlnet.de", "http://web2.altmuehlnet.de/fschmid/index.html" );
    aboutData.addCredit("Rolf Magnus", I18N_NOOP("Parts of the PNG support are copied from his KFile plugin for png support."),
                        "ramagnus@kde.org" );
    aboutData.addCredit("Michael v.Ostheim", I18N_NOOP("Created the Gentoo Ebuild scripts for Krename."),
                        "MvOstheim@web.de", "http://www.vonostheim.de" );
    aboutData.addCredit("Brandon Low", I18N_NOOP("Some GCC 3.1 fixes for Gentoo."),
                        "lostlogic@gentoo.org", "http://www.gentoo.org" );
    aboutData.addCredit("Per yvind Karlsen", I18N_NOOP("Thanks for creating the Mandrake RPM"),
                        "peroyvind@delonic.no" );
    aboutData.addCredit("Daniele Medri", I18N_NOOP("Italian translation"), "madrid@linuxmeeting.net" );
    aboutData.addCredit("Stephan Johach", I18N_NOOP("Provided a gcc3.x namepsace patch"), "lucardus@onlinehome.de" );
    aboutData.addCredit("Michal Zugaro", I18N_NOOP("Provided the new preview and move features") , "michael.zugaro@college-de-france.fr" );
    aboutData.addCredit("Rene Gass", I18N_NOOP("Fixed problems with the spec file and contributed a SuSE 8.1 rpm"), "kde-devel@gmx.de" );
    
    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineArgs::addCmdLineOptions( options );
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    KApplication a;

    KImageIO::registerFormats();

    krename = new wizard();
    a.setMainWidget(krename);

    QCStringList optlist = args->getOptionList ( "r" );
    for (QCStringList::ConstIterator it=optlist.begin(); it!=optlist.end(); ++it)
        krename->addDir( QString((*it)) );

    for( i = 0; i < args->count(); i++)
        krename->addFile( args->url( i ) );

    if( !args->isSet( "preview" ) )
        krename->noRealTimePreview = true;

    /* Check if Krename
     * was started from root!
     */
    uid = geteuid();
    if( uid == 0 )
        KMessageBox::information( krename, i18n(
            "<b>Krename was started from root!</b><br>"
            "When started from root, Krename may damage your "
            "system if you do not know exactly what you are "
            "doing!"
        ), i18n("Error"), "KrenameRootWarning" );

/*
 * Activate this warning message for unstable development releases.
 */
/*    KMessageBox::sorry( krename, i18n(
    "<b>Warning !</b> This is a development release which may cause damage to your files!"
    "<br>Make backups before using krename." ));
*/

    args->clear();      // Free some memory
    krename->show();
    krename->updatePre();
    //logo->hide();
//    delete logo;
    return a.exec();
}

