/***************************************************************************
                          pluginloader.cpp  -  description
                             -------------------
    begin                : Sun Dec 30 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * Parts of this code are copied from
 * Franz Schmid's <Franz.Schmid@altmuehlnet.de>
 * Scribus.
 */

// Own includes
#include "pluginloader.h"
#include "batchrenamer.h"
#include "fileplugin.h"
#include "datetime.h"
#include "mydirplugin.h"
#include "permission.h"
#include "pictureplugin.h"
#include "commandplugin.h"

// QT includes
#include <qdir.h>
#include <qmap.h>
#include <qstring.h>
#include <qvaluelist.h>
#include <qwidget.h>

// KDE includes
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kstddirs.h>
#include <kservice.h>

PluginLoader::PluginLoader()
{
    addPlugin( new MyPermPlugin() );
    addPlugin( new MyDatePlugin() );
    addPlugin( new MyDirPlugin() );
    addPlugin( new PicturePlugin() );
    addPlugin( new CommandPlugin() );

    KConfig* config = kapp->config();
    config->setGroup("GUISettings");    
    if( config->readBoolEntry("LoadFilePlugins", true ) )
        loadFilePlugins();
}

PluginLoader::~PluginLoader()
{ }

void PluginLoader::addPlugin( Plugin* plugin )
{
    /*
     * Add a plugin which is compiled directly into
     * KRename. Support for this plugins will be dropped
     * in the near future!
     */

    PluginLibrary* listitem=new PluginLibrary;
    listitem->plugin = plugin;
    listitem->usePlugin = false;
    listitem->isActive = listitem->plugin->defaultVisible();
    libs.insert(listitem->plugin->getName(), listitem);
}

void PluginLoader::loadFilePlugins()
{
    KService::List list = KService::allServices();
    for( unsigned int i = 0; i < list.count(); i++ ) {
        KService* s = (KService*)list[i];
        if( !s->terminal() && s->type() == "Service" && s->hasServiceType( "KFilePlugin" ) ) {
            FilePlugin* kfileplugin = new FilePlugin( s );
            if( kfileplugin->isValid() ) {
                PluginLibrary* listitem = new PluginLibrary;
                listitem->plugin = kfileplugin;
                listitem->usePlugin = false;
                listitem->isActive = listitem->plugin->defaultVisible();
                libs.insert(s->desktopEntryPath(), listitem);

                qDebug( "+-- Adding plugin " + s->desktopEntryPath() );
            } else
                qDebug( "--- Droping " + s->desktopEntryPath() );
        }
    }
}
