/***************************************************************************
                          replacedialog.cpp  -  description
                             -------------------
    begin                : Sat Aug 18 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Own includes
#include "replacedialog.h"
#include "batchrenamer.h"

// QT includes
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qtooltip.h>

// KDE includes
#include <kapplication.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <klocale.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <kparts/componentfactory.h>
#include <kparts/part.h>
#include <kpushbutton.h>
#include <kregexpeditorinterface.h>

ReplaceDialog::ReplaceDialog( QValueList<replacestrings> r, QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    setCaption( i18n( "Find and Replace" ) );
    ReplaceDialogLayout = new QHBoxLayout( this, 11, 6);

    Layout1 = new QHBoxLayout( 0, 0, 6 );
    Layout3 = new QVBoxLayout( 0, 0, 6 );
    Layout4 = new QVBoxLayout( 0, 0, 6 );
    Layout2 = new QHBoxLayout( 0, 0, 6 );
    Layout6 = new QVBoxLayout( 0, 0, 6 );
    Layout7 = new QHBoxLayout( 0, 0, 6 );
    Layout8 = new QVBoxLayout( 0, 0, 6 );

    list = new KListView( this );
    list->addColumn( i18n("Find:") );
    list->addColumn( i18n("Replace With:") );
    list->addColumn( i18n("RegExp:") );
    list->setColumnWidthMode( 0, QListView::Manual );
    list->setColumnWidthMode( 1, QListView::Manual );
    list->setColumnWidthMode( 2, QListView::Manual );
    list->setColumnWidth( 2, 0 );
    list->setAllColumnsShowFocus( true );
    list->setSorting( -1 );

    TextLabel1 = new QLabel( this );
    TextLabel1->setText( i18n( "Find:" ) );
    TextLabel2 = new QLabel( this );
    TextLabel2->setText( i18n( "Replace With:" ) );

    text1 = new QLineEdit( this );
    text2 = new QLineEdit( this );
    checkReg = new QCheckBox( i18n( "Regular Expression" ), this );

    buttonRegEdit = new KPushButton( this );
    buttonRegEdit->setText( i18n( "&Edit" ) );
    buttonRegEdit->setEnabled( false );

    buttonAdd = new KPushButton( this );
    buttonAdd->setText( i18n( "&Add" ) );

    buttonRemove = new KPushButton( this );
    buttonRemove->setText( i18n( "&Remove" ) );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    buttonOK = new KPushButton( this );
    buttonOK->setText( i18n( "&OK" ) );
    buttonOK->setDefault( true );
    buttonOK->setIconSet( SmallIconSet( "ok" ) );

    buttonCancel = new KPushButton( this );
    buttonCancel->setText( i18n( "&Cancel" ) );
    
    buttonSave = new KPushButton( this );
    buttonSave->setText( i18n( "&Save Settings" ) );

    Layout2->addWidget( TextLabel1 );
    Layout2->addWidget( text1 );

    Layout4->addLayout( Layout2 );
    Layout4->addWidget( checkReg );

    Layout1->addWidget( TextLabel2 );
    Layout1->addWidget( text2 );

    Layout3->addLayout( Layout1 );
    Layout3->addWidget( buttonRegEdit );

    Layout6->addWidget( buttonAdd );
    Layout6->addWidget( buttonRemove );
    Layout6->addItem( spacer );
    Layout6->addWidget( buttonSave );
    Layout6->addItem( spacer );
    Layout6->addWidget( buttonOK );
    Layout6->addWidget( buttonCancel );

    Layout7->addLayout( Layout4 );
    Layout7->addLayout( Layout3 );

    Layout8->addLayout( Layout7 );
    Layout8->addWidget( list );

    ReplaceDialogLayout->addLayout( Layout8 );
    ReplaceDialogLayout->addLayout( Layout6 );

    text1->setFocus();

    connect( buttonOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( add() ) );
    connect( buttonRemove, SIGNAL( clicked() ), this, SLOT( remove() ) );
    connect( buttonSave, SIGNAL( clicked() ), this, SLOT( save() ) );
    connect( list, SIGNAL( selectionChanged( QListViewItem* ) ), this, SLOT( setSelected(QListViewItem* ) ) );
    connect( text2, SIGNAL( returnPressed() ), this, SLOT( add() ) );
    connect( text1, SIGNAL( returnPressed() ), this, SLOT( moveFocus() ) );
    connect( checkReg, SIGNAL( clicked() ), this, SLOT( enableRegEdit() ) );
    connect( buttonRegEdit, SIGNAL( clicked() ), this, SLOT( invokeRegEdit() ) );

    edit = false;

    for( unsigned int i = 0; i < r.count(); i++ ) {
        replacestrings rs = r[i];

        rs.find = BatchRenamer::unEscape( rs.find );

        KListViewItem *item = new KListViewItem( list, encode( rs.find ), encode( rs.replace ), QString("%1").arg(rs.reg ) );
        list->insertItem( item );
    }
}

ReplaceDialog::~ReplaceDialog()
{ }

void ReplaceDialog::add()
{
    if( text1->text().isEmpty() ) {
        KMessageBox::sorry( this, i18n( "Add a text that should be replaced !" ) );
        return;
    }

    QListViewItem* item = list->firstChild();
    while( item ) {
        if( item->text( 0 ) == text1->text() ) {
            KMessageBox::sorry( this, i18n( "You can't replace the same text twice !" ) );
            return;
        }
        item = item->nextSibling();
    }

    if( !edit ) {
        KListViewItem* item = new KListViewItem
                ( list, encode( text1->text() ), encode( text2->text() ), QString("%1").arg( checkReg->isChecked() ) );
        list->insertItem( item );
    } else {
//        ListBox1->changeItem( encode( text1->text() ), editIndex );
//        ListBox2->changeItem( encode( text2->text() ), editIndex );
//        regList->changeItem( QString( "%1" ).arg( checkReg->isChecked() ), editIndex );
        edit = false;
    }

    reset();
}

void ReplaceDialog::remove()
{
    QListViewItem* item = list->firstChild();
    while( item ) {
        if( item->isSelected() ) {
            delete item;
            reset();
            edit = false;
            return;
        } else
            item = item->nextSibling();
    }
}

void ReplaceDialog::setSelected( QListViewItem* item )
{
    checkReg->setChecked( item->text( 2 ).toInt() );
    buttonRegEdit->setEnabled( item->text( 2 ).toInt() );
    text1->setText( decode( item->text( 0 ) ) );
    text2->setText( decode( item->text( 1 ) ) );
    edit = true;
}

QValueList<replacestrings> ReplaceDialog::getList()
{
    QValueList<replacestrings> r;
    QListViewItem* item = list->firstChild();
    while( item ) {
        replacestrings n;
        n.find = BatchRenamer::doEscape( decode( item->text( 0 ) ) );
        n.replace = decode( item->text( 1 ) );
        n.reg = item->text( 2 ).toInt();

        r.append( n );

        item = item->nextSibling();
    }
    return r;
}

void ReplaceDialog::moveFocus()
{
    text2->setFocus();
}

void ReplaceDialog::enableRegEdit()
{
    buttonRegEdit->setEnabled( checkReg->isChecked() );
}

void ReplaceDialog::invokeRegEdit()
{
    bool installed = !KTrader::self()->query("KRegExpEditor/KRegExpEditor").isEmpty();
    if( installed ) {
        QDialog *editorDialog = KParts::ComponentFactory::createInstanceFromQuery<QDialog>( "KRegExpEditor/KRegExpEditor" );
        if ( editorDialog ) {
            KRegExpEditorInterface *editor = dynamic_cast<KRegExpEditorInterface *>( editorDialog );

            QRegExp reg( text1->text() );
            if( reg.isValid() )
                // this line caused a crash!
                editor->setRegExp( text1->text() );

            if( editorDialog->exec() == QDialog::Accepted )
                text1->setText( editor->regExp() );
        }
    } else
        KMessageBox::information( this, i18n("Please install the kdeutils package to use the regular expressions editor.") );
}

void ReplaceDialog::save()
{
    QStringList find, replace;
    QValueList<int> regular;

    KConfig* conf = kapp->config();
    conf->setGroup("GUISettings");

    QListViewItem* item = list->firstChild();
    while( item ) {
        find.append( decode( item->text( 0 ) ) );
        replace.append( decode( item->text( 1 ) ) );
        regular.append( item->text( 2 ).toInt() );
        item = item->nextSibling();
    }

    conf->writeEntry( "FindList", find );
    conf->writeEntry( "ReplaceList", replace );
    conf->writeEntry( "RegularList", regular );

    conf->sync();
}

QString ReplaceDialog::encode( QString s )
{
    s.append("\"");
    s.prepend("\"");
    return s;
}

QString ReplaceDialog::decode( QString s )
{
    if( s[0] == '"' )
        s.remove( 0, 1 );

    if( s[s.length()-1] == '"' )
        s.remove( s.length()-1, 1 );
    return s;
}

void ReplaceDialog::resizeEvent( QResizeEvent* e )
{
    QDialog::resizeEvent( e );
    list->setColumnWidth( 0, TextLabel1->width() + text1->width() );
    list->setColumnWidth( 1, TextLabel2->width() + text2->width() );
}

void ReplaceDialog::reset()
{
    text1->clear();
    text2->clear();
    checkReg->setChecked( false );
    buttonRegEdit->setEnabled( false );
    text1->setFocus();
}

