/***************************************************************************
                         undodialog.cpp  -  description
                             -------------------
    begin                : Mon Mai 27 20:08:19 CEST 2002
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "undodialog.h"

// Qt includes
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

// KDE includes
#include <kfiledialog.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>
#include <kprocess.h>
#include <ktextbrowser.h>
#include <kurlrequester.h>

UndoDialog::UndoDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    resize( 458, 306 );
    setCaption( i18n( "Undo Renaming" ) );
    UndoDialogLayout = new QHBoxLayout( this, 11, 6, "UndoDialogLayout"); 

    Layout2 = new QVBoxLayout( 0, 0, 6, "Layout2"); 

    TextLabel1 = new QLabel( this, "TextLabel1" );
    TextLabel1->setText( i18n( "Undo Script:" ) );
    Layout2->addWidget( TextLabel1 );

    scriptname = new KURLRequester( this, "KURLRequester1" );
    scriptname->setMode( KFile::File | KFile::LocalOnly  );
    scriptname->fileDialog()->setOperationMode( KFileDialog::Opening );

    scriptname->setFilter( i18n("*.krename|KRename undo scripts (*.krename)\n"
                                   "*|All Files (*)") );
    Layout2->addWidget( scriptname );

    TextLabel2 = new QLabel( this, "TextLabel2" );
    TextLabel2->setText( i18n( "Undo Scripts are normal shell scripts<br>which can also be executed manually from the commandline." ) );
    Layout2->addWidget( TextLabel2 );

    browser = new KTextBrowser( this);
    browser->setWordWrap( QTextEdit::NoWrap );
    browser->setTextFormat( Qt::RichText );

    Layout2->addWidget( browser );
    UndoDialogLayout->addLayout( Layout2 );

    Layout1 = new QVBoxLayout( 0, 0, 6, "Layout1"); 

    buttonStart = new KPushButton( this, "buttonStart" );
    buttonStart->setText( i18n( "&Start" ) );
    buttonStart->setDefault( true );
    Layout1->addWidget( buttonStart );

    buttonClose= new KPushButton( this );
    buttonClose->setText( i18n( "&Close" ) );
    buttonClose->setIconSet( SmallIconSet("fileclose") );

    Layout1->addWidget( buttonClose );
    QSpacerItem* spacer = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Layout1->addItem( spacer );
    UndoDialogLayout->addLayout( Layout1 );

    connect( buttonClose, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( buttonStart, SIGNAL( clicked() ), this, SLOT( start() ) );
}

UndoDialog::~UndoDialog()
{ }

void UndoDialog::start()
{
    if( scriptname->url().right( 8 ) != ".krename" ) // EXTENSION
        if( KMessageBox::warningContinueCancel( this, i18n("This script does not seem "
                                        "to be a Krename undo script. Execution of this "
                                        "script can be dangerous. Continue ?") )
            == KMessageBox::Cancel )
            return;

    KProcess *proc = new KProcess;
    *proc << scriptname->url() << "--krename";

    buttonStart->setEnabled( false );

    if( !proc->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) {
        KMessageBox::sorry( this, i18n("Unable to start the given undo script!") );
        buttonStart->setEnabled( true );
        delete proc;
        return;
    }
    proc->resume();
    connect( proc, SIGNAL( receivedStdout( KProcess*, char*, int) ), this, SLOT( receive( KProcess*, char*, int ) ) );
    connect( proc, SIGNAL( receivedStderr( KProcess*, char*, int) ), this, SLOT( receiveErr( KProcess*, char*, int ) ) );
    connect( proc, SIGNAL( processExited( KProcess* ) ), this, SLOT( finished( KProcess* ) ) );
}

void UndoDialog::receive( KProcess* p, char* buffer, int len )
{
    QString text;
    for( int i = 0; i < len; i++ )
        text.append( buffer[i] );

    browser->setText( browser->text() + text + "<br>");
}

void UndoDialog::receiveErr( KProcess* p, char* buffer, int len )
{
    QString text = "<b>";
    for( int i = 0; i < len; i++ )
        text.append( buffer[i] );

    browser->setText( browser->text() + text + "</b><br>");
}

void UndoDialog::finished( KProcess* p )
{
    delete p;
    KMessageBox::information( this, i18n("Finished succesfully") );
    buttonStart->setEnabled( true );
}


